﻿using System.Collections.Generic;
using Windows.UI.Input;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Input;

namespace Chapter6
{
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            InitializeComponent();
        }

        // Keep track of which TextBlocks are used for which Pointers
        Dictionary<uint, TextBlock> textBlocks = new Dictionary<uint, TextBlock>();

        protected override void OnPointerPressed(PointerRoutedEventArgs e)
        {
            canvas.CapturePointer(e.Pointer);

            // Create a new TextBlock for this new Pointer
            TextBlock textBlock = new TextBlock
            {
                Text = e.Pointer.PointerId.ToString(),
                FontSize = 80
            };

            PlaceTextBlockUnderPointerPoint(textBlock, e.GetCurrentPoint(canvas));

            // Keep track of the TextBlock and add it to the canvas
            textBlocks[e.Pointer.PointerId] = textBlock;
            canvas.Children.Add(textBlock);
        }

        protected override void OnPointerMoved(PointerRoutedEventArgs e)
        {
            // Only do this if this Pointer is currently pressed
            if (textBlocks.ContainsKey(e.Pointer.PointerId))
            {
                // Retrieve the right TextBlock
                TextBlock textBlock = textBlocks[e.Pointer.PointerId];

                PlaceTextBlockUnderPointerPoint(textBlock, e.GetCurrentPoint(canvas));
            }
        }

        protected override void OnPointerReleased(PointerRoutedEventArgs e)
        {
            // Remove the TextBlock from the canvas and the dictionary
            if (textBlocks.ContainsKey(e.Pointer.PointerId))
            {
                canvas.Children.Remove(textBlocks[e.Pointer.PointerId]);
                textBlocks.Remove(e.Pointer.PointerId);
                canvas.ReleasePointerCapture(e.Pointer);
            }
        }

        protected override void OnPointerCanceled(PointerRoutedEventArgs e)
        {
            OnPointerReleased(e);
        }

        protected override void OnPointerCaptureLost(PointerRoutedEventArgs e)
        {
            OnPointerReleased(e);
        }

        void PlaceTextBlockUnderPointerPoint(TextBlock textBlock, PointerPoint point)
        {
            Canvas.SetLeft(textBlock, point.Position.X - (textBlock.ActualWidth / 2));
            Canvas.SetTop(textBlock, point.Position.Y - (textBlock.ActualHeight / 2));
        }
    }
}