﻿using System;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.FileProperties;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;
using PicturesLibraryViewer;

namespace Chapter17
{
    public sealed partial class MainPage : Page
    {
        // The new data source
        ObservableCollection<Photo> photos = new ObservableCollection<Photo>();

        public MainPage()
        {
            InitializeComponent();
            SetDataContext();
        }

        async Task SetDataContext()
        {
            // Can set this now because updates will be propagated
            this.DataContext = this.photos;

            // This requires the Pictures Library capability
            StorageFolder pictures = KnownFolders.PicturesLibrary;

            foreach (StorageFile file in await pictures.GetFilesAsync())
            {
                // Retreive the thumbnail
                StorageItemThumbnail thumbnail =
                  await file.GetThumbnailAsync(ThumbnailMode.PicturesView);

                if (thumbnail != null)
                {
                    using (thumbnail)
                    {
                        // Create an ImageSource from the thumbnail stream
                        BitmapImage source = new BitmapImage();
                        source.SetSource(thumbnail);

                        // Create a new Photo object and add it to the collection
                        this.photos.Add(new Photo { Thumbnail = source });
                    }
                }
            }
        }
    }
}