﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.FileProperties;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Data;

namespace Chapter17
{
    public sealed partial class MainPage : Page
    {
        // The new data source
        ObservableCollection<Photo> photos = new ObservableCollection<Photo>();

        public MainPage()
        {
            InitializeComponent();
            SetDataContext();
        }

        async Task SetDataContext()
        {
            // This requires the Pictures Library capability
            StorageFolder pictures = KnownFolders.PicturesLibrary;

            foreach (StorageFile file in await pictures.GetFilesAsync())
            {
                // Retreive the thumbnail
                StorageItemThumbnail thumbnail =
                  await file.GetThumbnailAsync(ThumbnailMode.PicturesView);
                if (thumbnail != null)
                {
                    using (thumbnail)
                    {
                        // Create an ImageSource from the thumbnail stream
                        BitmapImage source = new BitmapImage();
                        source.SetSource(thumbnail);

                        // Create a new Photo object and add it to the collection
                        this.photos.Add(new Photo
                        {
                            Thumbnail = source,
                            DateTaken =
                                (await file.Properties.GetImagePropertiesAsync()).DateTaken
                        });
                    }
                }
            }

            // Create a view to use as the data context
            CollectionViewSource viewSource = new CollectionViewSource();
            viewSource.IsSourceGrouped = true;
            this.DataContext = viewSource;

            // Here's where the underlying collection gets associated with the view. 
            // Use LINQ (requires "using System.Linq") to group the photos appropriately.
            viewSource.Source = from photo in this.photos
                                group photo by photo.DateTaken.Date into g
                                orderby g.Key
                                select g;
        }
    }
}