﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Media;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace BackgroundAudio
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        IRandomAccessStream stream;

        public MainPage()
        {
            this.InitializeComponent();

            MediaControl.AlbumArt = new Uri("ms-appx:///Assets/Logo.png");

            MediaControl.PlayPauseTogglePressed += OnPlayPauseTogglePressed;
            MediaControl.PlayPressed += OnPlayPressed;
            MediaControl.PausePressed += OnPausePressed;
            MediaControl.StopPressed += OnStopPressed;

            PlayUserSelectedMusic();
        }

        async void OnPlayPauseTogglePressed(object sender, object e)
        {
            // This is invoked on a background thread
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                // This must run on the main thread
                if (this.mediaElement.CurrentState == MediaElementState.Playing)
                {
                    this.mediaElement.Pause();
                }
                else
                {
                    this.mediaElement.Play();
                }
            });
        }

        async void OnPlayPressed(object sender, object e)
        {
            // This is invoked on a background thread
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                // This must run on the main thread
                this.mediaElement.Play();
            });
        }

        async void OnPausePressed(object sender, object e)
        {
            // This is invoked on a background thread
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                // This must run on the main thread
                this.mediaElement.Pause();
            });
        }

        async void OnStopPressed(object sender, object e)
        {
            // This is invoked on a background thread
            await this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                // This must run on the main thread
                this.mediaElement.Stop();
            });
        }

        async Task PlayUserSelectedMusic()
        {
            // Get an MP3 file from the user
            FileOpenPicker picker = new FileOpenPicker();
            picker.FileTypeFilter.Add(".mp3");
            picker.SuggestedStartLocation = PickerLocationId.MusicLibrary;
            StorageFile file = await picker.PickSingleFileAsync();
            if (file != null)
            {
                this.stream = await file.OpenAsync(FileAccessMode.Read);
                this.mediaElement.SetSource(this.stream, "audio/mp3");
                this.mediaElement.AudioCategory = AudioCategory.BackgroundCapableMedia;
            }
        }

        void MediaElement_MediaOpened(object sender, RoutedEventArgs e)
        {
            this.stream.Dispose();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }
    }
}
